"""File Change Actions."""

import logging
import pathlib
import shutil
import tempfile

import toml
import toml_sort

import templatise.configuration as _configuration
import templatise.licence as _licence

_LOGGER = logging.getLogger(__name__)


def convert_pyproject_toml(
    configuration: _configuration.Configuration, path: pathlib.Path
) -> None:
    """Convert pyproject.toml according to the configuration."""
    if path.name != "pyproject.toml":
        raise ValueError(
            f"convert_pyproject_toml is only for pyproject.toml files, not {path}; use convert_file instead."
        )

    pyproject_toml = toml.load(path)
    pyproject_toml["tool"]["poetry"]["name"] = configuration.sub(
        pyproject_toml["tool"]["poetry"]["name"]
    )
    pyproject_toml["tool"]["poetry"]["version"] = "0.1.0"
    pyproject_toml["tool"]["poetry"]["description"] = ""
    pyproject_toml["tool"]["poetry"]["authors"] = [
        configuration.sub(author)
        for author in pyproject_toml["tool"]["poetry"]["authors"]
    ]
    pyproject_toml["tool"]["poetry"]["repository"] = configuration.sub(
        pyproject_toml["tool"]["poetry"]["repository"]
    )
    pyproject_toml["tool"]["poetry"]["keywords"] = []
    pyproject_toml["tool"]["poetry"]["classifiers"] = [
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: The Unlicense (Unlicense)",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Typing :: Typed",
    ]
    del pyproject_toml["tool"]["poetry"]["scripts"]
    pyproject_toml["tool"]["pytest"]["ini_options"]["addopts"] = configuration.sub(
        pyproject_toml["tool"]["pytest"]["ini_options"]["addopts"]
    )
    pyproject_toml["tool"]["pytest"]["ini_options"]["testpaths"] = [
        configuration.sub(path)
        for path in pyproject_toml["tool"]["pytest"]["ini_options"]["testpaths"]
    ]
    pyproject_toml["tool"]["vulture"]["paths"] = [
        configuration.sub(path) for path in pyproject_toml["tool"]["vulture"]["paths"]
    ]

    pretty = toml_sort.TomlSort(
        toml.dumps(pyproject_toml),
        sort_config=toml_sort.tomlsort.SortConfiguration(inline_arrays=True),
    ).sorted()
    pretty = "\n".join(line.rstrip() for line in pretty.splitlines()).rstrip() + "\n"

    with tempfile.NamedTemporaryFile("w") as result:
        result.write(pretty)
        result.flush()
        shutil.copy(result.name, path)


def convert_file(
    configuration: _configuration.Configuration, path: pathlib.Path
) -> None:
    """Convert file according to the configuration."""
    if path.name == "pyproject.toml":
        raise ValueError(
            f"convert_file should not be used on {path}; use convert_pyproject_toml instead."
        )

    with tempfile.NamedTemporaryFile("w") as result:
        result.write(configuration.sub(path.read_text()))
        result.flush()
        shutil.copy(result.name, path)


def convert_module(
    configuration: _configuration.Configuration, path: pathlib.Path
) -> None:
    """Convert modules according to the configuration."""
    result = path.parent / configuration.sub(path.name) / "__init__.py"
    result.parent.mkdir(parents=True)
    result.touch()
    shutil.rmtree(path)


def convert_licence(
    configuration: _configuration.Configuration, path: pathlib.Path
) -> None:
    """Convert licence according to the configuration."""
    if (
        configuration.template is not None
        and configuration.licence != configuration.template.licence
    ):
        path.write_text(_licence.text(configuration.licence))



"""Project Name.

Convenience type for converting names to safe forms for different uses (e.g.,
project names).
"""

import dataclasses
import re
import typing


@dataclasses.dataclass
class ProjectName:
    """Project Name Type.

    Constructed with any arbitrary string and provides convenience properties
    for safe forms for other uses (e.g., package names).
    """

    original: str
    package: str

    def __init__(self, name: str, package_name: typing.Optional[str] = None):
        """Construct a ProjectName."""
        self.original = name
        assert isinstance(name, str)  # nosec
        self.package = _package_name(name) if not package_name else package_name


def _package_name(name: str) -> str:
    intercalated = []
    for character in name:
        if character.isupper():
            intercalated.append("_")
        intercalated.append(character)

    underscored = re.sub(r"\W+", "_", "".join(intercalated))
    return re.sub(r"__+", "_", underscored).lower()



"""Licence interactions."""
import datetime
import logging
import typing

import bs4
import requests
import retry

_LOGGER = logging.getLogger(__name__)


class InvalidError(RuntimeError):
    """Indicates an invalid licence was specified."""


class UnavailableError(RuntimeError):
    """Indicates licence information is not available."""


def text(name: str) -> str:
    """Full text for Licence with name."""
    response = _download(name)
    maybe_text = _extract(response.text)
    if maybe_text is None:
        raise RuntimeError(f"Could not find licence text in {response.url}.")
    return maybe_text


@retry.retry(UnavailableError, tries=2, logger=_LOGGER)
def _download(name: str) -> requests.Response:
    url = f"https://opensource.org/licenses/{name}"
    response = requests.get(url, timeout=datetime.timedelta(minutes=1).total_seconds())
    if response.status_code in range(400, 500):
        raise InvalidError(f"Requested {name}, but recieved {response}.")
    if response.status_code in range(500, 600):
        raise UnavailableError(f"Requested {name}, but received {response}.")

    return response


def _extract(html: str) -> typing.Optional[str]:
    soup = bs4.BeautifulSoup(html, features="html.parser")
    result = soup.find(name="div", id="LicenseText")
    if not result:
        result = soup.find(name="article")
    return result.text.strip() if result is not None else None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['templatise']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'click-log>=0.4.0,<0.5.0',
 'click>=8.1.3,<9.0.0',
 'requests>=2.28.1,<3.0.0',
 'retry>=0.9.2,<0.10.0',
 'toml-sort>=0.22.1,<0.23.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['initialise = templatise.initialise:main']}

setup_kwargs = {
    'name': 'templatise',
    'version': '1.0.0',
    'description': 'You can use template.py to create a new GitHub repository.',
    'long_description': '# template.py\n\n[Homepage][repository]\n\nBy Alex Brandt <alunduil@gmail.com>\n\n## Description\n\nYou can use template.py to create a new GitHub repository.  The repository will\nhave poetry, VS Code devcontainers, and various GitHub actions ready to use.\n\ntemplate.py is related to [Cookiecutter] because both are templates for\nbootstrapping projects.  template.py allows you to have a full development\nenvironment with only VS Code and it\'s "Remote Development" plugin.\n[Cookiecutter] expects you to re-use your development environment for multiple\nprojects.\n## Terms of use\n\nYou are free to use template.py as a basis for your own projects without any\nconditions.  See the [LICENSE] file for details.\n\n## Prerequisites\n\n1. VS Code with "Remote Development" installed\n\n## How to use this template\n\n1. Visit [the repository][repository]\n1. Click "Use this template"\n1. Follow the GitHub Docs to [Create a repo][create a repo]\n1. Open VS Code\n1. Open the command prompt (ctrl+shift+p)\n1. Type "clone repository in container" and hit return\n1. Input the GitHub URL of your new repository\n1. In the resulting terminal (ctrl+\\`), run: `poetry run initialise`\n1. Resolve the README update issue that is generated\n1. Continue working on your awesome project\n\n## Documentation\n\n* [LICENSE]: The license governing use of template.py\n\n## Getting Help\n\n* [GitHub Issues][issues]: Support requests, bug reports, and feature requests\n\n## How to Help\n\n* Submit [issues] for problems or questions\n* Submit [pull requests] for proposed changes\n\n[create a repo]: https://docs.github.com/en/get-started/quickstart/create-a-repo\n[issues]: https://github.com/alunduil/template.py/issues\n[LICENSE]: ./LICENSE\n[pull requests]: https://github.com/alunduil/template.py/pulls\n[repository]: https://github.com/alunduil/template.py\n[Cookiecutter]: https://github.com/cookiecutter/cookiecutter\n',
    'author': 'Alex Brandt',
    'author_email': 'alunduil@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/alunduil/template.py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
